using Microsoft.AspNetCore.Hosting;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Hosting;
using NetworkResurrector.Agent.Extensions.Serilog;
using NetworkResurrector.Agent.Extensions.Serilog.Sinks.MSSqlServer;
using Serilog;
using Serilog.Core;
using Serilog.Events;
using Serilog.Sinks.MSSqlServer;
using System;
using System.Diagnostics;
using System.IO;
using System.Linq;

namespace NetworkResurrector.Agent
{
    public class Program
    {
        public static void Main(string[] args)
        {
            var isConsole = Debugger.IsAttached || args.Contains("--console");
            if (!isConsole)
            {
                var pathToExe = Process.GetCurrentProcess().MainModule.FileName;
                var pathToContentRoot = Path.GetDirectoryName(pathToExe);
                Directory.SetCurrentDirectory(pathToContentRoot);
            }

            var configuration = new ConfigurationBuilder()
                .SetBasePath(Directory.GetCurrentDirectory())
                .AddJsonFile("appsettings.json", optional: false, reloadOnChange: true)
                .AddEnvironmentVariables()
                .Build();

            var connectionString = configuration.GetConnectionString("DatabaseConnection");
            var loggingLevelParam = configuration.GetValue<string>("Logging:LogLevel:Default");

            var loggingLevelOk = Enum.TryParse(loggingLevelParam, out LogEventLevel loggingLevel);
            if (!loggingLevelOk)
                throw new Exception($"Logging level '{loggingLevelParam}' is not valid.");

            var loggingLevelSwitch = new LoggingLevelSwitch(loggingLevel);

            var columnOptions = new ColumnOptions().AsStandard();
            var mssqlSinkOptions = new MSSqlServerSinkOptions() { AutoCreateSqlTable = true, TableName = "__Logs_Agent" };

            Log.Logger = new LoggerConfiguration()
                .MinimumLevel.ControlledBy(loggingLevelSwitch)
                .MinimumLevel.Override("Microsoft", LogEventLevel.Warning)
                .Enrich.FromLogContext()
                .Enrich.With(new AgentCodeEventEnricher(configuration))
                .WriteTo.Console()
                .WriteTo.MSSqlServer(connectionString, mssqlSinkOptions, columnOptions: columnOptions)
                .CreateLogger();

            try
            {
                var urls = configuration.GetValue<string>("Urls");
                var agentCode = configuration.GetValue<string>("Agent:Code");

                Log.Information($"Starting network resurrector agent {agentCode}...");
                Log.Information($"Network resurrector agent {agentCode} listening on {urls}");
                Log.Information($"Hostname: {Environment.MachineName}");
                Console.WriteLine("Application started. Press Ctrl+C to shut down.");
                CreateHostBuilder(args, configuration, !isConsole).Build().Run();
            }
            catch (Exception ex)
            {
                Log.Fatal(ex, "Network resurrector agent host terminated unexpectedly");
            }
            finally
            {
                Log.CloseAndFlush();
            }
        }

        public static IHostBuilder CreateHostBuilder(string[] args, IConfiguration configuration, bool useWindowsService)
        {
            var builder = Host.CreateDefaultBuilder(args)
                    .ConfigureWebHostDefaults(webBuilder =>
                    {
                        webBuilder.UseStartup<Startup>()
                            .UseConfiguration(configuration)
                            .UseSerilog();
                    });

            if (useWindowsService)
                builder.UseWindowsService();

            return builder;
        }
    }
}
