using MediatR;
using MediatR.Pipeline;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using NDB.Extensions.Swagger;
using NDB.Extensions.Swagger.Constants;
using NDB.Infrastructure.DatabaseMigration;
using NDB.Infrastructure.DatabaseMigration.Constants;
using NDB.Security.Authentication.Identity;
using NetworkResurrector.Agent.Wrapper;
using NetworkResurrector.Api.Application;
using NetworkResurrector.Api.Domain.Abstractions;
using NetworkResurrector.Api.Domain.Data;
using NetworkResurrector.Api.Services;
using NetworkResurrector.Server.Wrapper;
using Newtonsoft.Json;
using System.Reflection;

namespace NetworkResurrector.Api
{
    public class Startup
    {
        private readonly IConfiguration _configuration;

        public Startup(IConfiguration configuration)
        {
            _configuration = configuration;
        }

        // This method gets called by the runtime. Use this method to add services to the container.
        public void ConfigureServices(IServiceCollection services)
        {
            services.AddControllers()
                .AddNewtonsoftJson(o => o.SerializerSettings.DateTimeZoneHandling = DateTimeZoneHandling.Utc);

            // Add basic authentication 
            services.AddIdentityAuthentication(_configuration.GetSection("IdentityServer")["BaseAddress"]);

            services.AddHttpContextAccessor();
            services.AddScoped<IUserService, UserService>();

            // MediatR
            services.AddMediatR(GetMediatRAssemblies());
            services.AddScoped(typeof(IPipelineBehavior<,>), typeof(RequestPreProcessorBehavior<,>));
            services.AddScoped(typeof(IPipelineBehavior<,>), typeof(RequestPostProcessorBehavior<,>));

            // AutoMapper
            services.AddAutoMapper(typeof(Application.Mappings.MappingProfile).Assembly);

            // Swagger
            services.AddSwagger("NetworkResurrector API", AuthorizationType.InhouseIdentity);

            // Add network resurrector server services
            services.UseResurrectorServices(_configuration.GetSection("NetworkResurrectorServer")["BaseAddress"]);

            // Add network resurrector agent services
            services.UseResurrectorAgentServices();

            // Data access
            services.AddMigration(DatabaseType.SQLServer);
            services.AddDataAccess();

            // Application
            services.AddApplicationServices();
        }

        private Assembly[] GetMediatRAssemblies()
        {
            var assembly = typeof(Application.Queries.GetMachines).Assembly;
            return new Assembly[] { assembly };
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
        {
            // global cors policy
            app.UseCors(x => x.AllowAnyOrigin().AllowAnyMethod().AllowAnyHeader());
            app.UseExceptionHandler("/error");

            app.UseRouting();
            app.UseAuthentication();
            app.UseAuthorization();
            app.UseEndpoints(endpoints =>
            {
                endpoints.MapControllers();
            });
            app.ConfigureSwagger("NetworkResurrector API");

            app.UseMigration();
        }
    }
}
